import java.awt.*;
import javax.swing.*;
public class Ball extends Thread {
      private JPanel box; // referencia para o Panel onde sera feito o desenho
      private int x, y;   // coordenadas atuais das bolas
      private int dx, dy; // deslocamento em cada movimento
      private static final int XSIZE = 20; // tamanho na direcao x
      private static final int YSIZE = 20; // tamanho na direcao y
      private BouncingBalls bb;

      public Ball(JPanel box, int dx, int dy, BouncingBalls bb) {
	  this.box = box;
	  this.dx = dx;
	  this.dy = dy;
	  this.bb = bb;
      }
      public void setX(int x) { this.x = x; }
      public void setY(int y) { this.y = y; }
      public int getX() { return x; }
      public int getY() { return y; }
      public void draw() {
	  // pega a referencia do objeto grafico do panel
	  Graphics g = box.getGraphics();
	  g.fillOval(x, y, XSIZE, YSIZE);
	  g.dispose();
      }
      
      public void move() {
	  // pega a referencia do objeto Graphics do panel
	  Graphics g = box.getGraphics();
	  // limpa a area anterior da bola
	  g.clearRect(x, y, XSIZE, YSIZE);
	  // modifica as variaveis das coordenadas
	  x += dx;
	  y += dy;
	  // a partir do tamanho da janela, verifica se a 
	  // bola ultrapassou os limites e corrige
	  Dimension d = box.getSize();
	  if (x < 0) { 
	      x = 0;
	      dx = -dx;
	  }
	  if (y < 0) { 
	      y = 0;
	      dy = -dy;
	  }
	  if (x + XSIZE >= d.width) {
	      x = d.width - XSIZE;
	      dx = -dx;
	  }
	  if (y + YSIZE >= d.height) {
	      y = d.height - YSIZE;
	      dy = -dy;
	  }
	  // seleciona a cor e desenha a bola
	  g.setColor(Color.BLACK); // Color.GREEN // Color.RED
	  g.fillOval(x, y, XSIZE, YSIZE);
	  g.dispose();
      }
      public void bounce() throws InterruptedException {
	  // pega a trava do objeto bb
	  synchronized (bb) {
	    // caso o limite for alcancado
	    // desenha a bola e a thread aguarda notificacao
	    while (bb.reachedLimit()) {
		this.draw();
		bb.wait();
	    }
	  }
	  // envia mensagem a bb para
	  // informar que iniciou movimento
	  bb.startedBouncing();
	  for (int i = 0; i < 500; i++) {
	      move();
	      Thread.sleep(10);
	  }
      }
      public void run() {
	  draw();
	  while (true) {	      
	      try {
		// chama funcao que movimenta a bola 500 vezes
		bounce();
		// apos terminar de movimentar, 
		// pega a trava do objeto bb
		synchronized (bb) {
		    // envia mensagem a bb informando
		    // que finalizou o movimento
		    bb.stoppedBouncing();    
		    // notifica para permitir que outra threads
		    // saia do estado 'waiting'
		    bb.notify();
		}    
		// aguarda 10 milissegundos
		Thread.sleep(10);
	      } 
	      catch (InterruptedException e) {}
	  }
      }


} 
